/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.admin;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.Serializable;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import java.util.StringTokenizer;

import sync4j.framework.engine.source.ContentType;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceInfo;

import sync4j.syncadmin.AdminException;
import sync4j.syncadmin.ui.SourceManagementPanel;

import sync4j.exchange.engine.source.ExchangeCalendarSyncSource;
import sync4j.exchange.engine.source.ExchangeContactSyncSource;
import sync4j.exchange.engine.source.ExchangeNoteSyncSource;
import sync4j.exchange.engine.source.ExchangeTaskSyncSource;
import sync4j.exchange.engine.source.ExchangeSyncSource;

import org.apache.commons.lang.StringUtils;

/**
 * This class implements the configuration panel for an ExchangeSyncSource
 *
 * @author Luigia Fassina
 *
 * @version $Id: ExchangeSyncSourceConfigPanel.java,v 1.11 2005/06/03 13:45:14 stefano_fornari Exp $
 */
public class ExchangeSyncSourceConfigPanel
extends SourceManagementPanel
implements Serializable {

  // ----------------------------------------------------------------- Constants

  public static final String NAME_ALLOWED_CHARS =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_."     ;
  public static final String PORT_ALLOWED_CHARS =
    "0123456789"                                                            ;

  public static final String VCARD_TYPE         = "text/x-vcard";
  public static final String VCARD_TYPES        = "text/x-vcard,text/vcard" ;
  public static final String VCARD_VERSIONS     = "2.1, 3.0"                ;
  public static final String ICAL_TYPE          = "text/x-vcalendar"        ;
  public static final String ICAL_TYPES         = "text/x-vcalendar"        ;
  public static final String ICAL_VERSIONS      = "1.0"                     ;
  public static final String SIFC_TYPE          = "text/x-s4j-sifc"         ;
  public static final String SIFC_TYPES         = "text/x-s4j-sifc"         ;
  public static final String SIFC_VERSIONS      = "1.0"                     ;
  public static final String SIFE_TYPE          = "text/x-s4j-sife"         ;
  public static final String SIFE_TYPES         = "text/x-s4j-sife"         ;
  public static final String SIFE_VERSIONS      = "1.0";
  public static final String SIFN_TYPE          = "text/x-s4j-sifn"         ;
  public static final String SIFN_TYPES         = "text/x-s4j-sifn"         ;
  public static final String SIFN_VERSIONS      = "1.0";
  public static final String SIFT_TYPE          = "text/x-s4j-sift"         ;
  public static final String SIFT_TYPES         = "text/x-s4j-sift"         ;
  public static final String SIFT_VERSIONS      = "1.0"                     ;

  // -------------------------------------------------------------- Private data
  /** label for the panel's name */
  private JLabel             panelName = new JLabel();

  /** border to evidence the title of the panel */
  private TitledBorder       titledBorder1;

  private JLabel             nameLabel            = new JLabel        () ;
  private JTextField         nameValue            = new JTextField    () ;

  private JLabel             sourceUriLabel       = new JLabel        () ;
  private JTextField         sourceUriValue       = new JTextField    () ;

  private JLabel             exchangeFolderLabel  = new JLabel        () ;
  private JTextField         exchangeFolderValue  = new JTextField    () ;

  private JLabel             hostLabel            = new JLabel        () ;
  private JTextField         hostValue            = new JTextField    () ;

  private JLabel             portLabel            = new JLabel        () ;
  private JTextField         portValue            = new JTextField    () ;

  private JLabel             conversionTypeLabel  = new JLabel        () ;

  private JComboBox          conversion           = new JComboBox     () ;

  private JButton            confirmButton        = new JButton       () ;

  // ----------------------------------------------------------- Constructors

  public ExchangeSyncSourceConfigPanel() {
      init();
  }

  // ----------------------------------------------------------- Private methods
  /**
   * Create the panel
   * @throws Exception if error occures during creation of the panel
   */

  private void init(){
    // set layout
    this.setLayout(null);

    // set properties of label, position and border
    //  referred to the title of the panel
    titledBorder1 = new TitledBorder("");

    panelName.setFont(titlePanelFont);
    panelName.setText("Edit Exchange SyncSource");
    panelName.setBounds(new Rectangle(14, 5, 316, 28));
    panelName.setAlignmentX(SwingConstants.CENTER);
    panelName.setBorder(titledBorder1);

    sourceUriLabel.setText("Source URI: ");
    sourceUriLabel.setFont(defaultFont);
    sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 18)        );
    sourceUriValue.setFont(new java.awt.Font("Arial", 0, 12)       );
    sourceUriValue.setBounds(new Rectangle(170, 60, 350, 18)       );

    nameLabel.setText("Name: ");
    nameLabel.setFont(defaultFont);
    nameLabel.setBounds(new Rectangle(14, 90, 150, 18)             );
    nameValue.setFont(new java.awt.Font("Arial", 0, 12)            );
    nameValue.setBounds(new Rectangle(170, 90, 350, 18)            );

    hostLabel.setText("Exchange Server: ");
    hostLabel.setFont(defaultFont);
    hostLabel.setBounds(new Rectangle(14, 120, 150, 18)            );
    hostValue.setFont(new java.awt.Font("Arial", 0, 12)            );
    hostValue.setBounds(new Rectangle(170, 120, 350, 18)           );

    portLabel.setText("Exchange Port: ");
    portLabel.setFont(defaultFont);
    portLabel.setBounds(new Rectangle(14, 150, 150, 18)            );
    portValue.setFont(new java.awt.Font("Arial", 0, 12)            );
    portValue.setBounds(new Rectangle(170, 150, 350, 18)           );

    exchangeFolderLabel.setText("Exchange Folder: ");
    exchangeFolderLabel.setFont(defaultFont);
    exchangeFolderLabel.setBounds(new Rectangle(14, 180, 150, 18)  );
    exchangeFolderValue.setFont(new java.awt.Font("Arial", 0, 12)  );
    exchangeFolderValue.setBounds(new Rectangle(170, 180, 350, 18) );

    conversionTypeLabel.setText("Type: ");
    conversionTypeLabel.setFont(defaultFont);
    conversionTypeLabel.setBounds(new Rectangle(14, 210, 150, 18)  );
    conversion.setFont(new java.awt.Font("Arial", 0, 12)   );
    conversion.setBounds(new Rectangle(170, 210, 80, 18)   );

    confirmButton.setFont(defaultFont);
    confirmButton.setText("Add");
    confirmButton.setBounds(170, 270, 70, 25);

    confirmButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event ) {
            try {
                validateValues();
                getValues();
                if (getState() == STATE_INSERT) {
                    ExchangeSyncSourceConfigPanel.this.actionPerformed(new ActionEvent(ExchangeSyncSourceConfigPanel.this, ACTION_EVENT_INSERT, event.getActionCommand()));
                } else {
                    ExchangeSyncSourceConfigPanel.this.actionPerformed(new ActionEvent(ExchangeSyncSourceConfigPanel.this, ACTION_EVENT_UPDATE, event.getActionCommand()));
                }
            } catch (Exception e) {
                notifyError(new AdminException(e.getMessage()));
            }
        }
    });

    // add all components to the panel
    this.add(panelName            , null ) ;
    this.add(sourceUriLabel       , null ) ;
    this.add(sourceUriValue       , null ) ;
    this.add(nameLabel            , null ) ;
    this.add(nameValue            , null ) ;
    this.add(hostLabel            , null ) ;
    this.add(hostValue            , null ) ;
    this.add(portLabel            , null ) ;
    this.add(portValue            , null ) ;
    this.add(exchangeFolderLabel  , null ) ;
    this.add(exchangeFolderValue  , null ) ;
    this.add(conversionTypeLabel  , null ) ;
    this.add(conversion           , null ) ;
    this.add(confirmButton        , null ) ;

  }

  public void updateForm() {
      if (!(getSyncSource() instanceof ExchangeSyncSource)) {
          notifyError(
              new AdminException(
                  "This is not an ExchangeSyncSource! Unable to process SyncSource values."
              )
          );
          return;
      }
      
      ExchangeSyncSource syncSource = 
          (ExchangeSyncSource)getSyncSource();

      if (getState() == STATE_INSERT) {
          confirmButton.setText("Add");
      } else if (getState() == STATE_UPDATE) {
          confirmButton.setText("Save");
      }

      sourceUriValue.setText      (syncSource.getSourceURI()           );
      exchangeFolderValue.setText (syncSource.getExchangeFolder()      );
      nameValue.setText           (syncSource.getName()                );
      hostValue.setText           (syncSource.getHost()                );
      portValue.setText           (String.valueOf(syncSource.getPort()));

      conversion.removeAllItems();

      if (syncSource instanceof ExchangeCalendarSyncSource)  {
        conversion.addItem ("SIF-E" );
        conversion.addItem ("iCal"  );
      } else if (syncSource instanceof ExchangeContactSyncSource) {
        conversion.addItem ("SIF-C" );
        conversion.addItem ("vCard" );
      } else if (syncSource instanceof ExchangeNoteSyncSource) {
        conversion.addItem ("SIF-N" );
      } else if (syncSource instanceof ExchangeTaskSyncSource) {
        conversion.addItem ("SIF-T" );
      }

      if ((syncSource instanceof ExchangeCalendarSyncSource) ||
          (syncSource instanceof ExchangeContactSyncSource)) {

          String cType = ((ExchangeSyncSource) syncSource).getType();

          if (VCARD_TYPE.equals(cType) || ICAL_TYPE.equals(cType)) {
              conversion.setSelectedIndex(1);
          } else {
              conversion.setSelectedIndex(0);
          }
      }

      if (syncSource.getSourceURI() != null) {
          sourceUriValue.setEditable(false);
      }
  }


  // ----------------------------------------------------------- Private methods

  /**
   * Checks if the values provided by the user are all valid. In caso of errors,
   * a IllegalArgumentException is thrown.
   *
   * @throws IllegalArgumentException if:
   *         <ul>
   *         <li>name, uri, type or directory are empty (null or zero-length)
   *         <li>the types list length does not match the versions list length
   *         </ul>
   */
  private void validateValues() throws IllegalArgumentException {
      String value = null;

      value = nameValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Name' cannot be empty. "    +
                "Please provide a SyncSource name." );
      }

      if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are "   +
                "allowed for field 'Name': \n"         +
                NAME_ALLOWED_CHARS                     );
      }

      value = exchangeFolderValue.getText();
      if (StringUtils.isEmpty(value)) {
        throw new
            IllegalArgumentException(
                "Field 'Exchange folder' cannot be empty." +
                "Please provide an Exchange folder."       );
      }

      value = hostValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
          IllegalArgumentException(
              "Field 'Host' cannot be empty. "     +
              "Please provide a SyncSource host."  );
      }

      value = portValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Port' cannot be empty. "     +
                "Please provide a SyncSource port."  );
      }

      if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are " +
                "allowed for field 'Port': \n"       +
                PORT_ALLOWED_CHARS                   );
      }

      value = sourceUriValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new
            IllegalArgumentException(
                "Field 'Source URI' cannot be empty. "  +
                "Please provide a SyncSource URI."      );
      }

  }

  /**
   * Set syncSource properties with the values provided by the user.
   */
  private void getValues() {
      
        ExchangeSyncSource syncSource = (ExchangeSyncSource)getSyncSource();

        StringTokenizer types    = null ;
        StringTokenizer versions = null ;

        syncSource.setSourceURI
            (sourceUriValue.getText             ().trim() ) ;
        syncSource.setExchangeFolder
            (exchangeFolderValue.getText        ().trim() ) ;
        syncSource.setName
            (nameValue.getText                  ().trim() ) ;
        syncSource.setHost
            (hostValue.getText                  ().trim() ) ;
        syncSource.setPort
            (Integer.parseInt(portValue.getText ().trim())) ;

        if (conversion.getSelectedIndex() == 0)                               {
             syncSource.setEncode (true) ;

            if (syncSource instanceof ExchangeCalendarSyncSource)        {
                syncSource.setType(SIFE_TYPE);
                types = new StringTokenizer(SIFE_TYPES       , "," ) ;
                versions = new StringTokenizer(SIFE_VERSIONS , "," ) ;
            } else if (syncSource instanceof ExchangeContactSyncSource ) {
                syncSource.setType(SIFC_TYPE);
                types = new StringTokenizer(SIFC_TYPES       , "," ) ;
                versions = new StringTokenizer(SIFC_VERSIONS , "," ) ;
            } else if (syncSource instanceof ExchangeTaskSyncSource )    {
                syncSource.setType(SIFT_TYPE);
                types = new StringTokenizer(SIFT_TYPES       , "," ) ;
                versions = new StringTokenizer(SIFT_VERSIONS , "," ) ;
            } else if (syncSource instanceof ExchangeNoteSyncSource )    {
                syncSource.setType(SIFN_TYPE);
                types = new StringTokenizer(SIFN_TYPES       , "," ) ;
                versions = new StringTokenizer(SIFN_VERSIONS , "," ) ;
            }

        } else if (syncSource instanceof ExchangeCalendarSyncSource)     {
            syncSource.setEncode (false) ;
            syncSource.setType(ICAL_TYPE);
            types = new StringTokenizer(ICAL_TYPES       , "," ) ;
            versions = new StringTokenizer(ICAL_VERSIONS , "," ) ;
        } else if (syncSource instanceof ExchangeContactSyncSource )     {
            syncSource.setEncode (false ) ;
            syncSource.setType(VCARD_TYPE);
            types = new StringTokenizer(VCARD_TYPES       , "," ) ;
            versions = new StringTokenizer(VCARD_VERSIONS , "," ) ;
        }

        ContentType[] contentTypes= new ContentType[types.countTokens()];

        for(int i = 0, l = contentTypes.length; i < l; ++i) {
          contentTypes[i] = new ContentType(types.nextToken().trim()    ,
                                            versions.nextToken().trim() );
        }

        syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));
  }
}