/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.httptransport;

import java.io.*;

import java.net.Socket;

/**
 * This class provide methods
 * to access exchange server datastore
 * by socket connection
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: WebDavHttpTransport.java,v 1.8 2005/05/20 10:19:34 luigiafassina Exp $
 **/
public class WebDavHttpTransport {

    //---------------------------------------------------------------- Constants

    private static final int SIZE_INPUT_BUFFER = 4096;
    private static final String NEW_LINE = "\r\n" ;


    //---------------------------------------------------------------- Private data

    private String  host  = null ;
    private int     port  = 0    ;

    //---------------------------------------------------------------- Costructors

    public WebDavHttpTransport(String host,
                               int    port) {

        this.host = host ;
        this.port = port ;
    }

    //---------------------------------------------------------------- Public methods

    /**
     * Send request to exchange server
     * and get response.
     *
     * @param webDavCommand the first row of the header
     * @param credentials the credential
     * @param request the request
     * @param encoding the encoding to use to encode  the request and the response
     * @return response
     *
     * @throws IOException
     **/
    public String sendRequest (String webDavCommand,
                               String credentials,
                               String request,
                               String encoding)

        throws IOException {

        Socket               s         = null ;
        OutputStream         out       = null ;
        InputStream          in        = null ;

        StringBuffer temp = new StringBuffer(request);
        temp.append(NEW_LINE).append(NEW_LINE);
        request = temp.toString();

        StringBuffer requestToSend = new StringBuffer(webDavCommand);
        requestToSend.append("Host: ").append(host).append(NEW_LINE);
        requestToSend.append("Content-Type: text/xml; charset=\"").append(encoding).append("\"").append(NEW_LINE);
        requestToSend.append("Content-Length: ").append(request.getBytes(encoding).length).append(NEW_LINE);
        requestToSend.append("Authorization: Basic ").append(credentials).append(NEW_LINE);
        requestToSend.append(NEW_LINE);

        requestToSend.append(request);

        try {

            byte[] bRequest = requestToSend.toString().getBytes(encoding);

            s   = new Socket(this.host,
                             this.port) ;

            in  = s.getInputStream()  ;
            out = s.getOutputStream() ;

            out.write(bRequest);
            out.flush();

            byte[] bResponse = readContent(in);
            in.close();

            return new String(bResponse, encoding);

        } finally {

            if (out != null) {
                out.close () ;
            }
            if (in != null)  {
                in.close  () ;
            }
            if (s != null)  {
                s.close  ()  ;
            }
        }

    }

    /**
     * Reads the content from the inputstream.
     *
     * @param in the request InputStream
     *
     * @throws IOException in case of errors
     */
    protected byte[] readContent(final InputStream in) throws IOException {
        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        byte[] buf = new byte[SIZE_INPUT_BUFFER];

        int c = 0;
        int b = 0;
        while ((c < buf.length) && (b = in.read(buf, c, buf.length-c)) >= 0) {
            c+=b;
            if (c == SIZE_INPUT_BUFFER) {
                bout.write(buf);
                buf = new byte[SIZE_INPUT_BUFFER];
                c = 0;
            }
        }
        if (c != 0) {
            bout.write(buf, 0, c);
        }
        return bout.toByteArray();
    }


}
