/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.calendar.manager;

import java.io.IOException;

import sync4j.exchange.items.calendar.dao.CalendarDAO;
import sync4j.exchange.items.calendar.model.Calendar;
import sync4j.exchange.DataAccessException;
import java.util.Date;

/*
 * This class defines methods to access calendars data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: CalendarManager.java,v 1.7 2005/05/20 10:19:34 luigiafassina Exp $
 *
 **/
public class CalendarManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    CalendarDAO cd = null ;

    //------------------------------------------------------------- Costructor

    public CalendarManager(String exchangeServerHost ,
                           int    exchangeServerPort)
        throws DataAccessException {

        this.cd = new CalendarDAO (exchangeServerHost ,
                                   exchangeServerPort);

    }

    //------------------------------------------------------------- Public methods

    /**
     * get all calendars
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find calendars
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar[] getAllCalendars(String   username    ,
                                      String   credentials ,
                                      String   exchangeFolder )
        throws DataAccessException {

        return this.cd.getCalendars(username      ,
                                    credentials   ,
                                    new String[0] ,
                                    exchangeFolder     );
    }

    /**
     * get calendar twin
     *
     * @param calendar the calendar object
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar getCalendarTwin(Calendar calendar,
                                    String username,
                                    String credentials,
                                    String exchangeFolder)
    throws DataAccessException {

        Calendar[] calendars = null;

        Date dateStart = calendar.getDateStart();
        Date dateEnd = calendar.getDateEnd();
        String location = calendar.getLocation();

        String[] fields = {
                          CalendarDAO.TAG_DATE_START,
                          CalendarDAO.TAG_DATE_END,
                          CalendarDAO.TAG_LOCATION
        };

        Object[] values = {
                          dateStart,
                          dateEnd,
                          location
        };

        calendars = this.cd.getCalendars(username,
                                         credentials,
                                         fields,
                                         values,
                                         exchangeFolder);

        if (calendars == null || !(calendars.length > 0)) {
            return null;
        }

        return calendars[0];
    }


    /**
     * get calendars
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find calendars
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar[] getCalendars(String   username    ,
                                   String   credentials ,
                                   String[] ids         ,
                                   String   exchangeFolder   )
        throws DataAccessException {

         return this.cd.getCalendars(username    ,
                                     credentials ,
                                     ids         ,
                                     exchangeFolder   );

    }

    /**
     * get calendar by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find calendar
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar getCalendarById(String username    ,
                                    String credentials ,
                                    String id          ,
                                    String exchangeFolder   )
        throws DataAccessException {

        Calendar[] calendars = null;

        calendars = this.cd.getCalendars(username          ,
                                         credentials       ,
                                         new String[] {id} ,
                                         exchangeFolder);

        if (calendars == null || !(calendars.length > 0)) {
            return null;
        }

        return calendars[0];

    }

    /**
     * add calendar
     *
     * @param calendar
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new calendar
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Calendar setCalendar(Calendar calendar    ,
                                String   username    ,
                                String   credentials ,
                                String   exchangeFolder
                              )
        throws DataAccessException {

        return this.cd.setCalendar(calendar    ,
                                   username    ,
                                   credentials ,
                                   exchangeFolder   );

    }

    /**
     * remove calendar
     *
     * @param calendar
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeCalendar(Calendar calendar  ,
                               String   username  ,
                               String   principal ,
                               String   exchangeFolder )
        throws DataAccessException {

        this.cd.removeCalendar(calendar, username, principal, exchangeFolder);

    }

}
