/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.calendar.model;

import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.logging.Logger;
import java.util.logging.Level;

import java.util.Date;
import java.util.StringTokenizer;

import sync4j.framework.logging.Sync4jLogger;

import sync4j.exchange.items.calendar.CalendarParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Calendar.java,v 1.14 2005/05/11 15:58:54 nichele Exp $
 **/

public class Calendar {

    //--------------------------------------------------------------- Constants

    private static final String TAG_PP_DATE_START             = "Start"       ;
    private static final String TAG_PP_TIMEZONE_ID            = "TimezoneId"  ;
    private static final String TAG_PP_DATE_END               = "End"         ;
    private static final String TAG_PP_DURATION               = "Duration"    ;
    private static final String TAG_ALL_DAY_EVENT             = "AllDayEvent" ;
    private static final String TAG_PP_SUBJECT                = "Subject"     ;
    private static final String TAG_PP_LOCATION               = "Location"    ;
    private static final String TAG_PP_NOTE
           = "Body"                                                           ;
    private static final String TAG_PP_BUSY_STATUS            = "BusyStatus"  ;
    private static final String TAG_PP_MEETING_STATUS
           = "MeetingStatus"                                                  ;

    private static final String DEVICE_BUSYSTATUS_FREE        = "FREE"        ;
    private static final String DEVICE_BUSYSTATUS_BUSY        = "BUSY"        ;
    private static final String DEVICE_BUSYSTATUS_TENTATIVE   = "TENTATIVE"   ;
    private static final String DEVICE_BUSYSTATUS_OOF         = "OOF"         ;
    private static final String DEVICE_ALLDAYEVENT_TRUE       = "True"        ;
    private static final String DEVICE_ALLDAYEVENT_FALSE      = "False"       ;
    private static final String EXCHANGE_BUSYSTATUS_FREE      = "0"           ;
    private static final String EXCHANGE_BUSYSTATUS_BUSY      = "2"           ;
    private static final String EXCHANGE_BUSYSTATUS_TENTATIVE = "1"           ;
    private static final String EXCHANGE_BUSYSTATUS_OOF       = "3"           ;
    private static final String EXCHANGE_ALLDAYEVENT_TRUE     = "1"           ;
    private static final String EXCHANGE_ALLDAYEVENT_FALSE    = "0"           ;

    private static final String MSG_PP                   =
            "<appointment>\n"                            +
            "<Subject>{0}</Subject>\n"                   +
            "<Location>{1}</Location>\n"                 +
            "<Body>{2}</Body>\n"                         +
            "<BusyStatus>{3}</BusyStatus>\n"             +
            "<MeetingStatus>{4}</MeetingStatus>\n"       +
            "<TimezoneId>{5}</TimezoneId>\n"             +
            "<Start>{6}</Start>\n"                       +
            "<End>{7}</End>\n"                           +
            "<AllDayEvent>{9}</AllDayEvent>\n"           +
            "<Sequence>{10}</Sequence>\n"                +
            "</appointment>\n"                           ;

    //------------------------------------------------------------ Private data

    private String     id                  = null  ;
    private String     username            = null  ;
    private String     subject             = null  ;
    private String     textDescription     = null  ;
    private String     location            = null  ;

    private Date       dateStart           = null  ;
    private Date       dateEnd             = null  ;
    private long       duration            = 0     ;
    private Date       createDate          = null  ;
    private Date       lastModifiedDate    = null  ;

    private String     busyStatus          = null  ;
    private String     meetingStatus       = null  ;

    private int        timezoneId          = 0     ;

    private int        allDayEvent         = 0     ;
    private int        sequence            = 0     ;

    private String     href                = null  ;

    private Date       lastModified        = null  ;
    private char       state               = ' '   ;

    Logger  log = Sync4jLogger.getLogger("source");


    //------------------------------------------------------------ Constructors

    /**
     * Creates a Calendar
     *
     */
    public Calendar () {

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     *
     */
    public Calendar (String id) {

        this.id = id   ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public Calendar (String id, Date t)
    throws CalendarParseException {

        StringTokenizer  st             = null ;

        String           record         = null ;
        String           value          = null ;

        int              startPosition  = 0    ;
        int              endPosition    = 0    ;

        this.id                         = id   ;
        this.lastModified               = t    ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Calendar (String id, char state) {

        this.id      = id     ;
        this.state   = state  ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     *
     */
    public Calendar (String     id    ,
                     String     href  ,
                     char       state ,
                     Date       lastModified) {

        this.id            = id            ;
        this.href          = href          ;
        this.state         = state         ;
        this.lastModified  = lastModified  ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public Calendar (String id, String content, Date t)
    throws CalendarParseException {

        getPPCalendar    (id, content, t);

    }

    //---------------------------------------------------------- Public methods

    public String getId                ()  {
        return this.id;
    }

    public String getUsername          ()  {
            return this.username;
    }

    public String getSubject           ()  {
        return this.subject;
    }

    public String getTextDescription   ()  {
        return this.textDescription;
    }

    public int getTimezoneId           ()  {
        return this.timezoneId;
    }

    public String getLocation          ()  {
        return this.location;
    }

    public Date getDateStart           ()  {
        return this.dateStart;
    }

    public Date getDateEnd             ()  {
        return this.dateEnd;
    }

    public long getDuration            ()  {
        return this.duration;
    }

    public Date getCreateDate          ()  {
        return this.createDate;
    }

    public Date getLastModifiedDate    ()  {
        return this.lastModifiedDate;
    }

    public String getBusyStatus        ()  {
        return this.busyStatus;
    }

    public String getMeetingStatus     ()  {
        return this.meetingStatus;
    }

    public int getAllDayEvent          ()  {
        return this.allDayEvent;
    }

    public int getSequence             ()  {
            return this.sequence;
    }

    public Date getLastModified        ()  {
        return this.lastModified;
    }

    public char getState               ()  {
        return this.state;
    }

    public String getHref              ()  {
        return this.href;
    }


    public void setId(String id)                               {
        this.id = id;
    }

    public void setUsername(String username)                   {
        this.username = username;
    }

    public void setSubject(String subject)                     {
        this.subject = subject;
    }

    public void setTextDescription(String textDescription)     {
        this.textDescription = textDescription;
    }

    public void setLocation(String location)                   {
        this.location = location;
    }

    public void setTimezoneId(int timezoneId)                  {
        this.timezoneId = timezoneId;
    }

    public void setDateStart(Date dateStart)                   {
        this.dateStart = dateStart;
    }

    public void setDateEnd(Date dateEnd)                       {
        this.dateEnd = dateEnd;
    }

    public void setDuration(long duration)                     {
        this.duration = duration;
    }

    public void setCreateDate(Date createDate)                 {
        this.createDate = createDate;
    }

    public void setLastModifiedDate(Date lastModifiedDate)     {
        this.lastModifiedDate = lastModifiedDate;
    }

    public void setBusyStatus(String busyStatus)               {
        this.busyStatus = busyStatus;
    }

    public void setMeetingStatus(String meetingStatus)         {
        this.meetingStatus = meetingStatus;
    }

    public void setAllDayEvent(int allDayEvent)                {
        this.allDayEvent = allDayEvent;
    }

    public void setSequence(int sequence)                      {
        this.sequence = sequence;
    }

    public void setLastModified(Date lastModified)             {
        this.lastModified = lastModified;
    }

    public void setState(char state)                           {
        this.state = state;
    }

    public void setHref(String href)                           {
        this.href = href;
    }

    /**
     * Set a Calendar properties from Client format content
     *
     *
     * @param id client id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public void getPPCalendar (String id      ,
                               String content ,
                               Date   t       )
    throws CalendarParseException {

        String  tmz        = null ;
        String  dtStart    = null ;
        String  dtEnd      = null ;
        String  ade        = null ;

        String  drt        = null ;

        this.id            = id ;
        this.lastModified  = t   ;

        content = XmlParser.replaceTagDelimiter(content);

        try {

            XmlParser p = new XmlParser();

            tmz      = p.getXMLInitTagValue(content, TAG_PP_TIMEZONE_ID ) ;

            dtStart  = p.getXMLInitTagValue(content, TAG_PP_DATE_START  ) ;

            dtEnd    = p.getXMLInitTagValue(content, TAG_PP_DATE_END    ) ;

            drt      = p.getXMLInitTagValue(content, TAG_PP_DURATION    ) ;

            ade      = p.getXMLInitTagValue(content, TAG_ALL_DAY_EVENT  ) ;

            if (ade != null && DEVICE_ALLDAYEVENT_TRUE.equals(ade)) {
                ade = EXCHANGE_ALLDAYEVENT_TRUE  ;
            } else {
                ade = EXCHANGE_ALLDAYEVENT_FALSE ;
            }

            dateStart     = p.clientDateToDate (dtStart );

            dateEnd       = p.clientDateToDate (dtEnd   );

            if (tmz != null && tmz.trim().length() > 0) {
                timezoneId    = Integer.parseInt  (tmz);
            }

            if (drt != null && drt.length () > 0) {
                duration      = (Long.parseLong (drt)) * 60;
            } else {
                duration = ((dateEnd.getTime() - dateStart.getTime()) /
                            (1000));
            }

            if (ade != null && ade.length () > 0) {
                allDayEvent   = Integer.parseInt  (ade);
            } else {
                allDayEvent   = 0;
            }

            textDescription
                = p.getXMLInitTagValue  (content, TAG_PP_NOTE                    );

            subject       = p.getXMLInitTagValue (content, TAG_PP_SUBJECT        );


            //
            // work around for phone, as nokia 7650,
            // which sends textDescription as subject
            //
            if (subject == null || subject.length() == 0) {
                subject         = textDescription ;
                textDescription = ""              ;
            }

            location      = p.getXMLInitTagValue (content, TAG_PP_LOCATION       );

            busyStatus    = p.getXMLInitTagValue (content, TAG_PP_BUSY_STATUS    );

            if (this.busyStatus != null &&
                EXCHANGE_BUSYSTATUS_BUSY.equals(this.busyStatus))       {
                    busyStatus = DEVICE_BUSYSTATUS_BUSY;
            } else if (this.busyStatus != null &&
                EXCHANGE_BUSYSTATUS_FREE.equals(this.busyStatus))       {
                    busyStatus = DEVICE_BUSYSTATUS_FREE;
            } else if (this.busyStatus != null &&
                EXCHANGE_BUSYSTATUS_TENTATIVE.equals(this.busyStatus))  {
                    busyStatus = DEVICE_BUSYSTATUS_TENTATIVE;
            } else if (this.busyStatus != null &&
                EXCHANGE_BUSYSTATUS_OOF.equals(this.busyStatus))        {
                    busyStatus = DEVICE_BUSYSTATUS_OOF;
            }

            meetingStatus = p.getXMLInitTagValue (content, TAG_PP_MEETING_STATUS );



        } catch (NumberFormatException e) {
            throw new CalendarParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new CalendarParseException(e.getMessage());
        }

    }

    /**
     * Set a Calendar properties to Client format content
     *
     * @return content in client format
     *
     */
    public String toPP()
    throws CalendarParseException {

         String ppMsg = null;

         String dtStart   = null ;
         String dtEnd     = null ;
         String bs        = null ;
         String ade       = null ;
         String tmz       = null ;

         XmlParser p = new XmlParser();

         try {

             dtStart = p.toClientDate (dateStart     ) ;

             dtEnd   = p.toClientDate (dateEnd       ) ;

             ade     = String.valueOf (allDayEvent   ) ;

             tmz     = String.valueOf (timezoneId    ) ;

             if (ade != null && ade.length() > 0 &&
                 EXCHANGE_ALLDAYEVENT_TRUE.equals(ade)) {
                 ade = DEVICE_ALLDAYEVENT_TRUE  ;
             } else {
                 ade = DEVICE_ALLDAYEVENT_FALSE ;
             }

             if (this.busyStatus != null &&
                 DEVICE_BUSYSTATUS_BUSY.equals(this.busyStatus.toUpperCase()))      {
                 bs = EXCHANGE_BUSYSTATUS_BUSY;
             } else if (this.busyStatus != null &&
                 DEVICE_BUSYSTATUS_FREE.equals(this.busyStatus.toUpperCase()))      {
                 bs = EXCHANGE_BUSYSTATUS_FREE;
             } else if (this.busyStatus != null &&
                 DEVICE_BUSYSTATUS_TENTATIVE.equals(this.busyStatus.toUpperCase())) {
                 bs = EXCHANGE_BUSYSTATUS_TENTATIVE;
             } else if (this.busyStatus != null &&
                 DEVICE_BUSYSTATUS_OOF.equals(this.busyStatus.toUpperCase()))       {
                 bs = EXCHANGE_BUSYSTATUS_OOF;
             }

             ppMsg = MessageFormat.format(MSG_PP,
                new Object[] {
                                p.getString(subject                  ) ,
                                p.getString(location                 ) ,
                                p.getString(textDescription          ) ,
                                p.getString(bs                       ) ,
                                p.getString(meetingStatus            ) ,
                                p.getString(tmz                      ) ,
                                p.getString(dtStart                  ) ,
                                p.getString(dtEnd                    ) ,
                                p.getString(ade                      ) ,
                                p.getString(String.valueOf(sequence) )

                              }
             );

         } catch (Exception e) {
             throw new CalendarParseException ("Error composing calendar Device: " +
                                               e.toString());
         }

         return ppMsg;

     }

}
