/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.exchange.items.common.manager;

import java.io.IOException;

import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.items.common.model.Item;
import sync4j.exchange.DataAccessException;

/*
 * This class defines methods to access commons items data
 * in database source and exchange datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: ItemManager.java,v 1.6 2005/01/19 11:37:48 fabius Exp $
 *
 **/
public class ItemManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    //------------------------------------------------------------- Private Data

    ItemDAO id = null ;

    //------------------------------------------------------------- Costructor

    public ItemManager ()
        throws DataAccessException {

        this.id = new ItemDAO ();

    }

    public ItemManager (String  host ,
                        int     port )
    throws DataAccessException {

        this.id = new ItemDAO (host, port);
    }

    //------------------------------------------------------------- Public methods

    /**
     * delete local items
     *
     * @param sourceURI
     * @param principal
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void deleteLocalItems(String sourceURI, String principal)
    throws DataAccessException {

         id.deleteLocalItems(sourceURI, principal);

    }

    /**
     * get all local items
     *
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getLocalItems (String sourceURI, String principal)
    throws DataAccessException {

         return id.getLocalItems (sourceURI, principal);

    }

    /**
     * add local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void addLocalItems (Item[] localItems ,
                               String sourceURI  ,
                               String principal  )
    throws DataAccessException {

         id.addLocalItems (localItems, sourceURI, principal);

    }

    /**
     * update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void updateLocalItems(Item[] localItems ,
                                 String sourceURI  ,
                                 String principal  )
    throws DataAccessException {

         this.deleteLocalItems (sourceURI, principal);
         this.addLocalItems    (localItems, sourceURI, principal);

    }

    /**
     * get all Exchange Server items
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find items
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Item[] getExchangeItems(String username       ,
                                   String credentials    ,
                                   String exchangeFolder )
    throws DataAccessException {

        return id.getAllExchangeItems(username    ,
                                      credentials ,
                                      exchangeFolder);
    }

}