/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.exchange.items.contact.model;

import java.util.StringTokenizer;
import java.util.Date;
import java.text.MessageFormat;
import java.text.ParseException;

import sync4j.exchange.items.contact.ContactParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Contact.java,v 1.12 2005/05/02 21:19:58 luigiafassina Exp $
 **/

public class Contact {

    //--------------------------------------------------------------------- Constants

    private static final String TAG_PP_FIRST_NAME
        = "FirstName"                                        ;
    private static final String TAG_PP_LAST_NAME
        = "LastName"                                         ;
    private static final String TAG_PP_MIDDLE_NAME
        = "MiddleName"                                       ;
    private static final String TAG_PP_NICK_NAME
        = "NickName"                                         ;
    private static final String TAG_PP_JOBTITLE
        = "JobTitle"                                         ;
    private static final String TAG_PP_ORGANIZATION
        = "CompanyName"                                      ;
    private static final String TAG_PP_HOME_STREET
        = "HomeAddressStreet"                                ;
    private static final String TAG_PP_HOME_CITY
        = "HomeAddressCity"                                  ;
    private static final String TAG_PP_HOME_STATE
        = "HomeAddressState"                                 ;
    private static final String TAG_PP_HOME_POSTAL_CODE
        = "HomeAddressPostalCode"                            ;
    private static final String TAG_PP_HOME_COUNTRY
        = "HomeAddressCountry"                               ;
    private static final String TAG_PP_OFFICE_STREET
        = "BusinessAddressStreet"                            ;
    private static final String TAG_PP_OFFICE_CITY
        = "BusinessAddressCity"                              ;
    private static final String TAG_PP_OFFICE_STATE
        = "BusinessAddressState"                             ;
    private static final String TAG_PP_OFFICE_POSTAL_CODE
        = "BusinessAddressPostalCode"                        ;
    private static final String TAG_PP_OFFICE_COUNTRY
        = "BusinessAddressCountry"                           ;
    private static final String TAG_PP_OTHER_STREET
        = "OtherAddressStreet"                               ;
    private static final String TAG_PP_OTHER_CITY
        = "OtherAddressCity"                                 ;
    private static final String TAG_PP_OTHER_STATE
        = "OtherAddressState"                                ;
    private static final String TAG_PP_OTHER_POSTAL_CODE
        = "OtherAddressPostalCode"                           ;
    private static final String TAG_PP_OTHER_COUNTRY
        = "OtherAddressCountry"                              ;
    private static final String TAG_PP_OTHER_PHONE
        = "OtherTelephoneNumber"                             ;
    private static final String TAG_PP_OFFICE_PHONE
        = "BusinessTelephoneNumber"                          ;
    private static final String TAG_PP_HOME_PHONE
        = "HomeTelephoneNumber"                              ;
    private static final String TAG_PP_MOBILE_PHONE
        = "MobileTelephoneNumber"                            ;
    private static final String TAG_PP_ASSISTANT_PHONE
        = "AssistantTelephoneNumber"                         ;
    private static final String TAG_PP_OFFICE_PHONE2
        = "Business2TelephoneNumber"                         ;
    private static final String TAG_PP_OFFICE_FAX
        = "BusinessFaxNumber"                                ;
    private static final String TAG_PP_CAR_PHONE
        = "CarTelephoneNumber"                               ;
    private static final String TAG_PP_HOME_PHONE2
        = "Home2TelephoneNumber"                             ;
    private static final String TAG_PP_HOME_FAX
        = "HomeFaxNumber"                                    ;
    private static final String TAG_PP_PAGER
        = "IstantMessenger"                                  ;
    private static final String TAG_PP_BUSINESS_HOME_PAGE
        = "WebPage"                                          ;
    private static final String TAG_PP_NOTE
        = "Body"                                             ;
    private static final String TAG_PP_DEPARTMENT
        = "Department"                                       ;
    private static final String TAG_PP_SPOUSE
        = "Spouse"                                           ;
    private static final String TAG_PP_MANAGER
        = "ManagerName"                                      ;
    private static final String TAG_PP_ASSISTANT
        = "AssistantName"                                    ;
    private static final String TAG_PP_PROFESSION
        = "Profession"                                       ;
    private static final String TAG_PP_FILEAS
        = "FileAs"                                           ;
    private static final String TAG_PP_EMAIL1
        = "Email1Address"                                    ;
    private static final String TAG_PP_EMAIL1_NAME
        = "email1Name"                                       ;
    private static final String TAG_PP_EMAIL2
        = "Email2Address"                                    ;
    private static final String TAG_PP_EMAIL2_NAME
        = "email2Name"                                       ;
    private static final String TAG_PP_EMAIL3
        = "Email3Address"                                    ;
    private static final String TAG_PP_EMAIL3_NAME
        = "email3Name"                                       ;

    private static final String MSG_PP                                   =
        "<contact>\n"                                                    +
        "<FirstName>{0}</FirstName>\n"                                   +
        "<MiddleName>{1}</MiddleName>\n"                                 +
        "<LastName>{2}</LastName>\n"                                     +
        "<Subject>{3}</Subject>\n"                                       +
        "<JobTitle>{4}</JobTitle>\n"                                     +
        "<CompanyName>{5}</CompanyName>\n"                               +
        "<HomeAddressStreet>{6}</HomeAddressStreet>\n"                   +
        "<HomeAddressCity>{7}</HomeAddressCity>\n"                       +
        "<HomeAddressState>{8}</HomeAddressState>\n"                     +
        "<HomeAddressPostalCode>{9}</HomeAddressPostalCode>\n"           +
        "<HomeAddressCountry>{10}</HomeAddressCountry>\n"                +
        "<BusinessAddressStreet>{11}</BusinessAddressStreet>\n"          +
        "<BusinessAddressCity>{12}</BusinessAddressCity>\n"              +
        "<BusinessAddressState>{13}</BusinessAddressState>\n"            +
        "<BusinessAddressPostalCode>{14}</BusinessAddressPostalCode>\n"  +
        "<BusinessAddressCountry>{15}</BusinessAddressCountry>\n"        +
        "<OtherAddressStreet>{16}</OtherAddressStreet>\n"                +
        "<OtherAddressCity>{17}</OtherAddressCity>\n"                    +
        "<OtherAddressState>{18}</OtherAddressState>\n"                  +
        "<OtherAddressPostalCode>{19}</OtherAddressPostalCode>\n"        +
        "<OtherAddressCountry>{20}</OtherAddressCountry>\n"              +
        "<BusinessTelephoneNumber>{21}</BusinessTelephoneNumber>\n"      +
        "<HomeTelephoneNumber>{22}</HomeTelephoneNumber>\n"              +
        "<MobileTelephoneNumber>{23}</MobileTelephoneNumber>\n"          +
        "<AssistantTelephoneNumber>{24}</AssistantTelephoneNumber>\n"    +
        "<Business2TelephoneNumber>{25}</Business2TelephoneNumber>\n"    +
        "<BusinessFaxNumber>{26}</BusinessFaxNumber>\n"                  +
        "<CarTelephoneNumber>{27}</CarTelephoneNumber>\n"                +
        "<Home2TelephoneNumber>{28}</Home2TelephoneNumber>\n"            +
        "<HomeFaxNumber>{29}</HomeFaxNumber>\n"                          +
        "<IstantMessenger>{30}</IstantMessenger>\n"                      +
        "<Email1Address>{31}</Email1Address>\n"                          +
        "<Email2Address>{32}</Email2Address>\n"                          +
        "<Email3Address>{33}</Email3Address>\n"                          +
        "<WebPage>{34}</WebPage>\n"                                      +
        "<Body>{35}</Body>\n"                                            +
        "<Department>{36}</Department>\n"                                +
        "<Spouse>{37}</Spouse>\n"                                        +
        "<AssistantName>{38}</AssistantName>\n"                          +
        "<FileAs>{39}</FileAs>\n"                                        +
        "<NickName>{40}</NickName>\n"                                    +
        "<Profession>{41}</Profession>\n"                                +
        "<ManagerName>{42}</ManagerName>\n"                              +
        "<OtherTelephoneNumber>{43}</OtherTelephoneNumber>\n"            +
        "<OtherFaxNumber>{44}</OtherFaxNumber>\n"                        +
        "</contact>\n"                                                   ;

    //--------------------------------------------------------------------- Private data

    private String   id                              = null ;

    private String   username                        = null ;

    private String   firstName                       = null ;
    private String   middleName                      = null ;
    private String   lastName                        = null ;

    private String   department                      = null ;
    private String   organization                    = null ;
    private String   office                          = null ;
    private String   jobtitle                        = null ;
    private String   profession                      = null ;
    private String   nickName                        = null ;
    private String   spouse                          = null ;
    private String   manager                         = null ;
    private String   assistant                       = null ;

    private String   homeStreet                      = null ;
    private String   homeCity                        = null ;
    private String   homeState                       = null ;
    private String   homePostalCode                  = null ;
    private String   homeCountry                     = null ;
    private String   homePhone                       = null ;
    private String   homeOtherPhone                  = null ;
    private String   homeFax                         = null ;

    private String   officeStreet                    = null ;
    private String   officeCity                      = null ;
    private String   officeState                     = null ;
    private String   officePostalCode                = null ;
    private String   officeCountry                   = null ;
    private String   officePhone                     = null ;
    private String   officeOtherPhone                = null ;
    private String   officeFax                       = null ;

    private String   otherStreet                     = null ;
    private String   otherCity                       = null ;
    private String   otherState                      = null ;
    private String   otherPostalCode                 = null ;
    private String   otherCountry                    = null ;
    private String   otherPhone                      = null ;
    private String   otherFax                        = null ;

    private String   callBackPhone                   = null ;
    private String   mobilePhone                     = null ;
    private String   mobileOtherPhone                = null ;
    private String   organizationPhone               = null ;
    private String   assistantPhone                  = null ;

    private String   internationalIsdnNumber         = null ;
    private String   pager                           = null ;

    private String   emailAddress1                   = null ;
    private String   emailDescription1               = null ;
    private String   emailAddress2                   = null ;
    private String   emailDescription2               = null ;
    private String   emailAddress3                   = null ;
    private String   emailDescription3               = null ;

    private String   businessHomePage                = null ;
    private String   burl                            = null ;
    private String   note                            = null ;

    private String   fileas                          = null ;

    private String   href                            = null ;

    private Date     lastModified                    = null ;
    private char     state                           = ' '  ;

    //------------------------------------------------------------ Constructors

    /**
     * Creates a Contact
     *
     */
    public Contact () {

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     *
     */
    public Contact (String id) {

        this.id = id   ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public Contact (String id, Date t) {

        this.id            = id   ;
        this.lastModified  = t    ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Contact (String id, char state) {

        this.id      = id     ;
        this.state   = state  ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Contact (String   id    ,
                    String   href  ,
                    char     state ,
                    Date     lastModified) {

        this.id            = id            ;
        this.href          = href          ;
        this.state         = state         ;
        this.lastModified  = lastModified  ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public Contact (String id, String content, Date t)
    throws ContactParseException {

        getPocketPcContact (id, content, t);

    }

    //----------------------------------------------------------------------- Public methods

    public String getId()                      {
        return this.id;
    }

    public String getUsername()                {
            return this.username;
    }

    public String getFirstName()               {
        return this.firstName;
    }

    public String getMiddleName()              {
        return this.middleName;
    }

    public String getLastName()                {
        return this.lastName;
    }

    public String getDepartment()              {
        return this.department;
    }

    public String getOffice()                  {
        return this.office;
    }

    public String getJobTitle()                {
        return this.jobtitle;
    }

    public String getProfession()              {
        return this.profession;
    }

    public String getOrganization()            {
        return this.organization;
    }

    public String getNickName()                {
        return this.nickName;
    }

    public String getSpouse()                  {
        return this.spouse;
    }

    public String getManager()                 {
        return this.manager;
    }

    public String getAssistant()                {
        return this.assistant;
    }

    public String getHomeStreet()              {
        return this.homeStreet;
    }

    public String getHomeCity()                {
        return this.homeCity;
    }

    public String getHomeState()               {
        return this.homeState;
    }

    public String getHomePostalCode()          {
        return this.homePostalCode;
    }

    public String getHomeCountry()             {
        return this.homeCountry;
    }

    public String getHomePhone()               {
        return this.homePhone;
    }

    public String getHomeOtherPhone()          {
        return this.homeOtherPhone;
    }

    public String getHomeFax()                 {
        return this.homeFax;
    }

    public String getOfficeStreet()            {
        return this.officeStreet;
    }

    public String getOfficeCity()              {
        return this.officeCity;
    }

    public String getOfficeState()             {
        return this.officeState;
    }

    public String getOfficePostalCode()        {
        return this.officePostalCode;
    }

    public String getOfficeCountry()           {
        return this.officeCountry;
    }

    public String getOfficePhone()             {
        return this.officePhone;
    }

    public String getOfficeOtherPhone()        {
        return this.officeOtherPhone;
    }

    public String getOfficeFax()               {
        return this.officeFax;
    }

    public String getOtherStreet()             {
            return this.otherStreet;
    }

    public String getOtherCity()               {
        return this.otherCity;
    }

    public String getOtherState()              {
        return this.otherState;
    }

    public String getOtherPostalCode()         {
        return this.otherPostalCode;
    }

    public String getOtherCountry()            {
        return this.otherCountry;
    }

    public String getOtherPhone()              {
        return this.otherPhone;
    }

    public String getOtherFax()                {
        return this.otherFax;
    }

    public String getCallBackPhone()           {
        return this.callBackPhone;
    }

    public String getMobilePhone()             {
        return this.mobilePhone;
    }

    public String getMobileOtherPhone()        {
        return this.mobileOtherPhone;
    }

    public String getOrganizationPhone()       {
        return this.organizationPhone;
    }

    public String getAssistantPhone()          {
        return this.assistantPhone;
    }

    public String getInternationalIsdnNumber() {
        return this.internationalIsdnNumber;
    }

    public String getPager()                   {
        return this.pager;
    }

    public String getEmailAddress1()           {
        return this.emailAddress1;
    }

    public String getEmailDescription1()       {
        return this.emailDescription1;
    }

    public String getEmailAddress2()           {
        return this.emailAddress2;
    }

    public String getEmailDescription2()       {
        return this.emailDescription2;
    }

    public String getEmailAddress3()           {
        return this.emailAddress3;
    }

    public String getEmailDescription3()       {
        return this.emailDescription3;
    }

    public String getBusinessHomePage()        {
        return this.businessHomePage;
    }

    public String getBurl()                    {
        return this.burl;
    }

    public String getNote()                    {
        return this.note;
    }

    public String getFileas()                  {
        return this.fileas;
    }

    public Date getLastModified()              {
        return this.lastModified;
    }

    public char getState()                     {
        return this.state;
    }

    public String getHref()                    {
        return this.href;
    }


    public void setId(String id)                                           {
        this.id = id;
    }

    public void setUsername(String username)                               {
            this.username = username;
    }

    public void setFirstName(String firstName)                             {
        this.firstName = firstName;
    }

    public void setMiddleName(String middleName)                           {
        this.middleName = middleName;
    }

    public void setLastName(String lastName)                               {
        this.lastName = lastName;
    }

    public void setDepartment(String department)                           {
        this.department = department;
    }

    public void setOffice(String office)                                   {
        this.office = office;
    }

    public void setJobTitle(String jobtitle)                               {
        this.jobtitle = jobtitle;
    }

    public void setProfession(String profession)                           {
        this.profession = profession;
    }

    public void setOrganization(String organization)                       {
        this.organization = organization;
    }

    public void setNickName(String nickName)                               {
        this.nickName = nickName;
    }

    public void setSpouse(String spouse)                                   {
        this.spouse = spouse;
    }

    public void setManager(String manager)                                 {
        this.manager = manager;
    }

    public void setAssistant(String assistant)                             {
        this.assistant = assistant;
    }

    public void setHomeStreet(String homeStreet)                           {
        this.homeStreet = homeStreet;
    }

    public void setHomeCity(String homeCity)                               {
        this.homeCity = homeCity;
    }

    public void setHomeState(String homeState)                             {
        this.homeState = homeState;
    }

    public void setHomePostalCode(String homePostalCode)                   {
        this.homePostalCode = homePostalCode;
    }

    public void setHomeCountry(String homeCountry)                         {
        this.homeCountry = homeCountry;
    }

    public void setHomePhone(String homePhone)                             {
        this.homePhone = homePhone;
    }

    public void setHomeOtherPhone(String homeOtherPhone)                   {
        this.homeOtherPhone = homeOtherPhone;
    }

    public void setHomeFax(String homeFax)                                 {
        this.homeFax = homeFax;
    }

    public void setOfficeStreet(String officeStreet)                       {
        this.officeStreet = officeStreet;
    }

    public void setOfficeCity(String officeCity)                           {
        this.officeCity = officeCity;
    }

    public void setOfficeState(String officeState)                         {
        this.officeState = officeState;
    }

    public void setOfficePostalCode(String officePostalCode)               {
        this.officePostalCode = officePostalCode;
    }

    public void setOfficeCountry(String officeCountry)                     {
        this.officeCountry = officeCountry;
    }

    public void setOfficePhone(String officePhone)                         {
        this.officePhone = officePhone;
    }

    public void setOfficeOtherPhone(String officeOtherPhone)               {
        this.officeOtherPhone = officeOtherPhone;
    }

    public void setOfficeFax(String officeFax)                             {
        this.officeFax = officeFax;
    }

    public void setOtherStreet(String otherStreet)                         {
        this.otherStreet = otherStreet;
    }

    public void setOtherCity(String otherCity)                             {
        this.otherCity = otherCity;
    }

    public void setOtherState(String otherState)                           {
        this.otherState = otherState;
    }

    public void setOtherPostalCode(String otherPostalCode)                 {
        this.otherPostalCode = otherPostalCode;
    }

    public void setOtherCountry(String otherCountry)                       {
        this.otherCountry = otherCountry;
    }

    public void setOtherPhone(String otherPhone)                           {
        this.otherPhone = otherPhone;
    }

    public void setOtherFax(String otherFax)                               {
        this.otherFax = otherFax;
    }

    public void setCallBackPhone(String callBackPhone)                     {
        this.callBackPhone = callBackPhone;
    }

    public void setMobilePhone(String mobilePhone)                         {
        this.mobilePhone = mobilePhone;
    }

    public void setMobileOtherPhone(String mobileOtherPhone)               {
        this.mobileOtherPhone = mobileOtherPhone;
    }

    public void setOrganizationPhone(String organizationPhone)             {
        this.organizationPhone = organizationPhone;
    }

    public void setAssistantPhone(String assistantPhone)                   {
        this.assistantPhone = assistantPhone;
    }

    public void setInternationalIsdnNumber(String internationalIsdnNumber) {
        this.internationalIsdnNumber = internationalIsdnNumber;
    }

    public void setPager(String pager)                                     {
        this.pager = pager;
    }

    public void setEmailAddress1(String emailAddress1)                     {
        this.emailAddress1 = emailAddress1;
    }

    public void setEmailDescription1(String emailDescription1)             {
        this.emailDescription1 = emailDescription1;
    }

    public void setEmailAddress2(String emailAddress2)                     {
        this.emailAddress2 = emailAddress2;
    }

    public void setEmailDescription2(String emailDescription2)             {
        this.emailDescription2 = emailDescription2;
    }

    public void setEmailAddress3(String emailAddress3)                     {
        this.emailAddress3 = emailAddress3;
    }

    public void setEmailDescription3(String emailDescription3)             {
        this.emailDescription3 = emailDescription3;
    }

    public void setBusinessHomePage(String businessHomePage)               {
        this.businessHomePage = businessHomePage;
    }

    public void setBurl(String burl)                                       {
        this.burl = burl;
    }

    public void setNote(String note)                                       {
        this.note = note;
    }

    public void setFileas(String fileas)                                   {
        this.fileas = fileas;
    }

    public void setLastModified(Date lastModified)                         {
        this.lastModified = lastModified;
    }

    public void setState(char state)                                       {
        this.state = state;
    }

    public void setHref(String href)                                       {
        this.href = href;
    }

    /**
     * Set a Contact properties from Client format content
     *
     *
     * @param id client id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public void getPocketPcContact (String id, String content, Date t)
    throws ContactParseException {

       String value  = null ;

       XmlParser p  = new XmlParser();

       this.id                   = id ;
       this.lastModified         = t  ;

       content = XmlParser.replaceTagDelimiter(content);

       try {

           this.firstName        = p.getXMLInitTagValue(content,
                                                        TAG_PP_FIRST_NAME     );
           this.lastName         = p.getXMLInitTagValue(content,
                                                        TAG_PP_LAST_NAME      );
           this.middleName       = p.getXMLInitTagValue(content,
                                                        TAG_PP_MIDDLE_NAME    );
           this.nickName         = p.getXMLInitTagValue(content,
                                                        TAG_PP_NICK_NAME      );
           this.jobtitle         = p.getXMLInitTagValue(content,
                                                        TAG_PP_JOBTITLE       );
           this.organization     = p.getXMLInitTagValue(content,
                                                        TAG_PP_ORGANIZATION   );

           this.homeStreet       = p.getXMLInitTagValue(content,
                                                        TAG_PP_HOME_STREET    );
           this.homeCity         = p.getXMLInitTagValue(content,
                                                        TAG_PP_HOME_CITY      );
           this.homeState        = p.getXMLInitTagValue(content,
                                                        TAG_PP_HOME_STATE     );
           this.homePostalCode   = p.getXMLInitTagValue(content,
                                               TAG_PP_HOME_POSTAL_CODE        );
           this.homeCountry      = p.getXMLInitTagValue(content,
                                               TAG_PP_HOME_COUNTRY            );

           this.officeStreet     = p.getXMLInitTagValue(content,
                                                        TAG_PP_OFFICE_STREET  );
           this.officeCity       = p.getXMLInitTagValue(content,
                                                        TAG_PP_OFFICE_CITY    );
           this.officeState      = p.getXMLInitTagValue(content,
                                                        TAG_PP_OFFICE_STATE   );
           this.officePostalCode = p.getXMLInitTagValue(content,
                                               TAG_PP_OFFICE_POSTAL_CODE      );
           this.officeCountry    = p.getXMLInitTagValue(content,
                                               TAG_PP_OFFICE_COUNTRY          );

           this.otherStreet      = p.getXMLInitTagValue(content,
                                                        TAG_PP_OTHER_STREET   );
           this.otherCity        = p.getXMLInitTagValue(content,
                                                        TAG_PP_OTHER_CITY     );
           this.otherState       = p.getXMLInitTagValue(content,
                                                        TAG_PP_OTHER_STATE    );
           this.otherPostalCode  = p.getXMLInitTagValue(content,
                                               TAG_PP_OTHER_POSTAL_CODE       );
           this.otherCountry     = p.getXMLInitTagValue(content,
                                               TAG_PP_OTHER_COUNTRY           );

           this.officePhone      = p.getXMLInitTagValue(content,
                                                        TAG_PP_OFFICE_PHONE   );
           this.homePhone        = p.getXMLInitTagValue(content,
                                                        TAG_PP_HOME_PHONE     );
           this.mobilePhone      = p.getXMLInitTagValue(content,
                                                        TAG_PP_MOBILE_PHONE   );
           this.assistantPhone   = p.getXMLInitTagValue(content,
                                                        TAG_PP_ASSISTANT_PHONE);
           this.officeOtherPhone = p.getXMLInitTagValue(content,
                                                        TAG_PP_OFFICE_PHONE2  );
           this.otherPhone       = p.getXMLInitTagValue(content,
                                                        TAG_PP_OTHER_PHONE    );
           this.officeFax        = p.getXMLInitTagValue(content,
                                                        TAG_PP_OFFICE_FAX     );
           this.mobileOtherPhone = p.getXMLInitTagValue(content,
                                                        TAG_PP_CAR_PHONE      );
           this.homeOtherPhone   = p.getXMLInitTagValue(content,
                                                        TAG_PP_HOME_PHONE2    );
           this.homeFax          = p.getXMLInitTagValue(content,
                                                        TAG_PP_HOME_FAX       );
           this.pager            = p.getXMLInitTagValue(content,
                                                        TAG_PP_PAGER          );

           this.businessHomePage = p.getXMLInitTagValue(content,
                                               TAG_PP_BUSINESS_HOME_PAGE     );
           this.note             = p.getXMLInitTagValue(content,
                                                        TAG_PP_NOTE          );
           this.department       = p.getXMLInitTagValue(content,
                                                        TAG_PP_DEPARTMENT    );
           this.spouse           = p.getXMLInitTagValue(content,
                                                        TAG_PP_SPOUSE        );
           this.manager          = p.getXMLInitTagValue(content,
                                                        TAG_PP_MANAGER       );
           this.assistant        = p.getXMLInitTagValue(content,
                                                        TAG_PP_ASSISTANT     );
           this.profession       = p.getXMLInitTagValue(content,
                                                        TAG_PP_PROFESSION    );

           value                 = p.getXMLInitTagValue(content,
                                                        TAG_PP_FILEAS        ) ;

           if (value == null || value.length() == 0) {

                this.fileas = "";

                if (this.firstName != null && this.firstName.length() > 0) {
                    this.fileas = firstName + ",";
                }

                if (this.lastName != null && this.lastName.length() > 0) {
                    this.fileas = this.fileas + " " + lastName;
                }

                if (this.middleName != null && this.middleName.length() > 0) {

                    if ((this.firstName != null && this.firstName.length() > 0) ||
                        (this.lastName != null && this.lastName.length() > 0)) {
                        this.fileas = this.fileas + " ";
                    }

                    this.fileas = this.fileas +  middleName;
                }

           } else {
               this.fileas = value;
           }


           value                     = p.getXMLInitTagValue(content,
                                                        TAG_PP_EMAIL1        ) ;

           if (value != null && value.length() > 0) {
                this.emailAddress1   = "\""                +
                                        this.firstName     +
                                        " "                +
                                        this.lastName      +
                                        "\"" + " &lt;"     +
                                        value              +
                                        "&gt;"             ;

           } else {
                this.emailAddress1 = null;
           }

           this.emailDescription1    = p.getXMLInitTagValue(content,
                                                        TAG_PP_EMAIL1_NAME   ) ;
           this.emailAddress2        = p.getXMLInitTagValue(content,
                                                        TAG_PP_EMAIL2        ) ;
           this.emailDescription2    = p.getXMLInitTagValue(content,
                                                        TAG_PP_EMAIL2_NAME   ) ;
           this.emailAddress3        = p.getXMLInitTagValue(content,
                                                        TAG_PP_EMAIL3        ) ;
           this.emailDescription3    = p.getXMLInitTagValue(content,
                                                        TAG_PP_EMAIL3_NAME   ) ;

        } catch (XmlParseException e) {
                    throw new ContactParseException ("Error composing contact xml: " +
                                                      e.getMessage());
        }

     }

     /**
      * Set a Contact properties to Client format content
      *
      * @return content in client format
      *
     */
     public String toPP()
     throws ContactParseException {

         String ppMsg = null;

         XmlParser p = new XmlParser();

          try {

             ppMsg = MessageFormat.format(MSG_PP,
                 new Object[] {
                                p.getString(firstName         ),
                                p.getString(middleName        ),
                                p.getString(lastName          ),
                                p.getString(lastName +
                                            " "      +
                                            firstName         ),
                                p.getString(jobtitle          ),
                                p.getString(organization      ),
                                p.getString(homeStreet        ),
                                p.getString(homeCity          ),
                                p.getString(homeState         ),
                                p.getString(homePostalCode    ),
                                p.getString(homeCountry       ),
                                p.getString(officeStreet      ),
                                p.getString(officeCity        ),
                                p.getString(officeState       ),
                                p.getString(officePostalCode  ),
                                p.getString(officeCountry     ),
                                p.getString(otherStreet       ),
                                p.getString(otherCity         ),
                                p.getString(otherState        ),
                                p.getString(otherPostalCode   ),
                                p.getString(otherCountry      ),
                                p.getString(officePhone       ),
                                p.getString(homePhone         ),
                                p.getString(mobilePhone       ),
                                p.getString(assistantPhone    ),
                                p.getString(officeOtherPhone  ),
                                p.getString(officeFax         ),
                                p.getString(mobileOtherPhone  ),
                                p.getString(homeOtherPhone    ),
                                p.getString(homeFax           ),
                                p.getString(pager             ),
                                p.getString(emailAddress1     ),
                                p.getString(emailAddress2     ),
                                p.getString(emailAddress3     ),
                                p.getString(businessHomePage  ),
                                p.getString(note              ),
                                p.getString(department        ),
                                p.getString(spouse            ),
                                p.getString(assistant         ),
                                p.getString(fileas            ),
                                p.getString(nickName          ),
                                p.getString(profession        ),
                                p.getString(manager           ),
                                p.getString(otherPhone        ),
                                p.getString(otherFax          )

                              }
             );

         } catch (Exception e) {
             throw new ContactParseException ("Error composing contact Device: " +
                                               e.getMessage());
         }

         return ppMsg;

     }

}