/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.exchange.items.note.manager;

import sync4j.exchange.items.note.dao.NoteDAO;
import sync4j.exchange.items.note.model.Note;
import sync4j.exchange.DataAccessException;

/*
 * This class defines methods to access notes data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: NoteManager.java,v 1.5 2005/01/19 11:37:48 fabius Exp $
 *
 **/
public class NoteManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    NoteDAO nd = null ;

    //------------------------------------------------------------- Costructor

    public NoteManager(String exchangeServerHost ,
                       int    exchangeServerPort )
        throws DataAccessException {

        this.nd = new NoteDAO (exchangeServerHost  ,
                               exchangeServerPort );

    }

    //------------------------------------------------------------- Public methods

    /**
     * get all notes
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find notes
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note[] getAllNotes(String   username       ,
                              String   credentials    ,
                              String   exchangeFolder )
        throws DataAccessException {

        return this.nd.getNotes(username       ,
                                credentials    ,
                                new String[0]  ,
                                exchangeFolder );
    }

    /**
     * get notes
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find notes
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note[] getNotes(String   username       ,
                           String   credentials    ,
                           String[] ids            ,
                           String   exchangeFolder )
        throws DataAccessException {

         return this.nd.getNotes(username       ,
                                 credentials    ,
                                 ids            ,
                                 exchangeFolder );

    }

    /**
     * get note by id
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find note
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note getNoteById(String username      ,
                            String credentials   ,
                            String id            ,
                            String exchangeFolder)
        throws DataAccessException {

        Note[] notes = null;

        notes = this.nd.getNotes(username          ,
                                 credentials       ,
                                 new String[] {id} ,
                                 exchangeFolder    );

        if (notes == null || !(notes.length > 0)) {
            return null;
        }

        return notes[0];

    }

    /**
     * add / update note
     *
     * @param note
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new note
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Note setNote(Note    note           ,
                        String  username       ,
                        String  credentials    ,
                        String  exchangeFolder )
        throws DataAccessException {

        return this.nd.setNote(note           ,
                               username       ,
                               credentials    ,
                               exchangeFolder );

    }

    /**
     * remove note
     *
     * @param note
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeNote(Note     note           ,
                           String   username       ,
                           String   principal      ,
                           String   exchangeFolder )
        throws DataAccessException {

        this.nd.removeNote(note, username, principal, exchangeFolder);

    }

}