/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.exchange.items.task.manager;

import java.io.IOException;

import sync4j.exchange.items.task.dao.TaskDAO;
import sync4j.exchange.items.task.model.Task;
import sync4j.exchange.DataAccessException;

/*
 * This class defines methods to access task data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: TaskManager.java,v 1.5 2005/01/19 11:37:48 fabius Exp $
 *
 **/
public class TaskManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    TaskDAO td = null ;

    //------------------------------------------------------------- Costructor

    public TaskManager(String exchangeServerHost ,
                       int    exchangeServerPort)
        throws DataAccessException {

        this.td = new TaskDAO (exchangeServerHost ,
                               exchangeServerPort);

    }

    //------------------------------------------------------------- Public methods

    /**
     * get all tasks
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find tasks
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task[] getAllTasks(String   username       ,
                              String   credentials    ,
                              String   exchangeFolder )
        throws DataAccessException {

        return this.td.getTasks(username       ,
                                credentials    ,
                                new String[0]  ,
                                exchangeFolder );
    }

    /**
     * get tasks
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find tasks
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task[] getTasks(String   username       ,
                           String   credentials    ,
                           String[] ids            ,
                           String   exchangeFolder )
        throws DataAccessException {

         return this.td.getTasks(username       ,
                                 credentials    ,
                                 ids            ,
                                 exchangeFolder );

    }

    /**
     * get task by id
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find task
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task getTaskById(String username       ,
                            String credentials    ,
                            String id             ,
                            String exchangeFolder )
        throws DataAccessException {

        Task[] tasks = null;

        tasks = this.td.getTasks(username          ,
                                 credentials       ,
                                 new String[] {id} ,
                                 exchangeFolder    );

        if (tasks == null || !(tasks.length > 0)) {
            return null;
        }

        return tasks[0];

    }

    /**
     * add / update task
     *
     * @param task
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new tsk
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task setTask(Task    task           ,
                        String  username       ,
                        String  credentials    ,
                        String  exchangeFolder )
        throws DataAccessException {

        return this.td.setTask(task           ,
                               username       ,
                               credentials    ,
                               exchangeFolder );

    }

    /**
     * remove task
     *
     * @param task
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeTask(Task     task           ,
                           String   username       ,
                           String   principal      ,
                           String   exchangeFolder )
        throws DataAccessException {

        this.td.removeTask(task, username, principal, exchangeFolder);

    }


}