/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.exchange.items.task.model;

import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.StringTokenizer;

import sync4j.exchange.items.task.TaskParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Task.java,v 1.11 2005/06/03 14:04:10 luigiafassina Exp $
 **/
public class Task {

    //---------------------------------------------------------------- Constants

    private static final String TAG_PP_DATE     = "StartDate";
    private static final String TAG_PP_SUBJECT  = "Subject"  ;
    private static final String TAG_PP_NOTE     = "Body"     ;
    private static final String TAG_PP_COMPLETE = "Complete" ;

    private static final String MSG_PP   =
          "<task>\n"                     +
          "<Subject>{0}</Subject>\n"     +
          "<Body>{1}</Body>\n"           +
          "<StartDate>{2}</StartDate>\n" +
          "<Complete>false</Complete>\n" +
          "</task>\n"                    ;

    //------------------------------------------------------------- Private data

    private String id              = null ;
    private String username        = null ;
    private String subject         = null ;
    private String textDescription = null ;
    private String href            = null ;
    private char   state           = ' '  ;
    private boolean complete       = false;

    private Date   date            = null;
    private Date   createDate      = null;
    private Date   lastModified    = null;
    
    //------------------------------------------------------------- Constructors

    /**
     * Creates a Task
     *
     */
    public Task() {
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     */
    public Task (String id) {
        this.id = id;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param t client item timestamp
     */
    public Task(String id, Date t) throws TaskParseException {
        this.id           = id;
        this.lastModified = t ;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param state client item state
     */
    public Task(String id, char state) {
        this.id    = id   ;
        this.state = state;
    }

    /**
     * Creates a Task
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     */
    public Task(String id          ,
                String href        ,
                char   state       ,
                Date   lastModified) {

        this.id           = id          ;
        this.href         = href        ;
        this.state        = state       ;
        this.lastModified = lastModified;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public Task(String id, String content, Date t) throws TaskParseException {
        getPPTask(id, content, t);
    }

    //----------------------------------------------------------- Public methods
    public String getId() {
        return this.id;
    }

    public String getUsername() {
        return this.username;
    }

    public String getSubject() {
        return this.subject;
    }

    public String getTextDescription() {
        return this.textDescription;
    }

    public Date getDate() {
        return this.date;
    }

    public String getHref() {
        return this.href;
    }

    public Date getLastModified() {
        return this.lastModified;
    }

    public char getState() {
        return this.state;
    }
    
    public boolean isComplete() {
        return this.complete;
    }
    
    public void setId(String id) {
        this.id = id;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public void setTextDescription(String textDescription) {
        this.textDescription = textDescription;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public void setHref(String href) {
        this.href = href;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public void setState(char state) {
        this.state = state;
    }

    public void setComplete(boolean complete) {
        this.complete = complete;
    }

    /**
     * Set a Task properties from Client format content
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public void getPPTask(String id, String content, Date t) 
    throws TaskParseException {

        String dt = null;

        this.id           = id;
        this.lastModified = t ;

        content = XmlParser.replaceTagDelimiter(content);

        try {

            XmlParser p = new XmlParser();
            dt = p.getXMLInitTagValue(content, TAG_PP_DATE );

            this.date            = p.clientDateToDate(dt);
            this.subject         = p.getXMLInitTagValue(content, TAG_PP_SUBJECT);
            this.textDescription = p.getXMLInitTagValue(content, TAG_PP_NOTE);

        } catch (NumberFormatException e) {
            throw new TaskParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new TaskParseException(e.getMessage());
        }
    }

    /**
     * Set a Task properties to Client format content
     *
     * @return content in client format
     */
    public String toPP() throws TaskParseException {

        String ppMsg = null;
        String date  = null;
        XmlParser p  = new XmlParser();

        try {

            date = p.toClientDate(this.date);
            ppMsg = MessageFormat.format(MSG_PP,
                new Object[] {
                    p.getString(subject         ) ,
                    p.getString(textDescription ) ,
                    p.getString(date            )
                }
            );

         } catch (Exception e) {
             throw new TaskParseException("Error composing task Device: " +
                                           e.getMessage());
         }

         return ppMsg;
     }
}